﻿Imports System.Windows.Forms.Screen

Public Class Form1
  Const KeywordNotSet = "(なし)"
  Const KeywordAddNewItem = "(新規...)"
  Const KeywordInputNewData = "新規入力"
  Const KeywordInputNewDataCancel = "新規入力中止"
  Private FilePath As String
  Private ColumnCount As Integer

  Private EditingNewDataValue As Boolean
  Private Property EditingNewData As Boolean
    Get
      Return EditingNewDataValue
    End Get
    Set(ByVal value As Boolean)
      EditingNewDataValue = value
      SetMoveButtons(Not value)
      EnableTextBoxAndComboBox(True)
      If DataGridViewMain.RowCount <> 0 Then _
        ButtonOverwrite.Enabled = Not value
      ButtonSaveAsNew.Enabled = True

      If value Then
        ButtonNew.Text = KeywordInputNewDataCancel
        ClearTabPageInput()
        TextBoxNameLast.Focus()
      Else
        ButtonNew.Text = KeywordInputNewData
        If DataGridViewMain.RowCount <> 0 Then _
          ShowOneRow(DataGridViewMain.CurrentRow.Index)
      End If
    End Set
  End Property

  Private Sub SetMoveButtons(ByVal enabled As Boolean)
    If enabled And DataGridViewMain.RowCount > 1 _
      And Not ShowSearched Then
      Select Case GetRowIndex()
        Case 0
          ButtonFirst.Enabled = False
          ButtonPrevious.Enabled = False
          ButtonLast.Enabled = True
          ButtonNext.Enabled = True
        Case DataGridViewMain.RowCount - 1
          ButtonFirst.Enabled = True
          ButtonPrevious.Enabled = True
          ButtonLast.Enabled = False
          ButtonNext.Enabled = False
        Case Else
          ButtonFirst.Enabled = True
          ButtonPrevious.Enabled = True
          ButtonNext.Enabled = True
          ButtonLast.Enabled = True
      End Select
    Else
      ButtonFirst.Enabled = False
      ButtonPrevious.Enabled = False
      ButtonNext.Enabled = False
      ButtonLast.Enabled = False
    End If
  End Sub

  Private Sub EnableTextBoxAndComboBox(
    ByVal value As Boolean)

    For Each c As Control In
      TabPageInput.Controls
      If TypeOf c Is TextBox Or
        TypeOf c Is ComboBox Then _
        c.Enabled = value
    Next
  End Sub

  Private Sub ClearTabPageInput()
    LabelId.Text = "#" & GetNewDataIndex()
    For Each c In TabPageInput.Controls
      If TypeOf c Is TextBox Then _
        CType(c, TextBox).Clear()
      If TypeOf c Is ComboBox Then _
        CType(c, ComboBox).SelectedItem =
        KeywordNotSet
    Next
  End Sub

  Private Function GetNewDataIndex() As Integer
    If DataGridViewMain.RowCount = 0 Then
      Return 1
    Else
      Dim index As Integer = 1
      For Each row As DataGridViewRow In
        DataGridViewMain.Rows
        If row.Cells(0).Value > index Then _
          index = row.Cells(0).Value
      Next
      Return index + 1
    End If
  End Function

  Private Property ShowMenu As Boolean
    Get
      Return My.Settings.ShowMenu
    End Get
    Set(ByVal value As Boolean)
      My.Settings.ShowMenu = value
      ToolStripMenuItemSettingShowMenu.
        Checked = value
      MenuStrip1.Visible = value
    End Set
  End Property

  Private Property AutoLoad As Boolean
    Get
      Return My.Settings.AutoLoad
    End Get
    Set(ByVal value As Boolean)
      My.Settings.AutoLoad = value
      ToolStripMenuItemSettingAutoLoad.
        Checked = value
    End Set
  End Property

  Private FileNameValue As String
  Private Property FileName As String
    Get
      Return FileNameValue
    End Get
    Set(ByVal value As String)
      FileNameValue = value
      If value <> "" Then FilePath =
        System.IO.Path.GetDirectoryName(value)
      Edited = False
    End Set
  End Property

  Private EditedValue As Boolean
  Private Property Edited As Boolean
    Get
      Return EditedValue
    End Get
    Set(ByVal value As Boolean)
      EditedValue = value
      UpdateStatus()
    End Set
  End Property

  Private Sub UpdateStatus()
    Dim s As String = Application.ProductName
    If FileName <> "" Then s &= " - " & FileName
    s &= " - " & DataGridViewMain.RowCount & "件"
    If Edited Then s &= "（変更あり）"
    Me.Text = s

    If FileName = "" Or Not Edited Or
      DataGridViewMain.RowCount = 0 Then
      ToolStripMenuItemFileSave.Enabled = False
    Else
      ToolStripMenuItemFileSave.Enabled = True
    End If

    If Not Edited Or
      DataGridViewMain.RowCount = 0 Then
      ToolStripMenuItemFileSaveAs.Enabled = False
    Else
      ToolStripMenuItemFileSaveAs.Enabled = True
    End If
  End Sub

  Enum SearchMode
    Id
    Group
    FreeWord
  End Enum

  Structure SearchParameter
    Public Mode As SearchMode
    Public Keyword As String
  End Structure
  Private LastSearch As SearchParameter

  Private ShowSearchedValue As Boolean
  Private Property ShowSearched As Boolean
    Get
      Return ShowSearchedValue
    End Get
    Set(ByVal value As Boolean)
      ShowSearchedValue = value
      If value Then
        CopyGridSetting(DataGridViewMain,
          DataGridViewSearch)
        TabControlMain.SelectedIndex = 0
      Else
        CopyGridSetting(DataGridViewSearch,
          DataGridViewMain)
      End If
      DataGridViewMain.Visible = Not value
      DataGridViewSearch.Visible = value
    End Set
  End Property

  Sub CopyGridSetting(
    ByVal source As DataGridView,
    ByVal target As DataGridView)

    For i As Integer = 0 To ColumnCount - 1
      target.Columns(i).Width =
        source.Columns(i).Width
    Next

    If Not source.CurrentRow Is Nothing Then
      Dim rowIndex = source.CurrentRow.Index
      If rowIndex >= 0 Then
        Dim id As String =
          source(0, rowIndex).Value
        For i As Integer = 0 To _
          target.RowCount - 1
          If id = target(0, i).Value Then
            target(0, i).Selected = True
            Exit For
          End If
        Next
      End If
    End If
  End Sub

  Private Sub Form1_Load(ByVal sender As System.Object,
    ByVal e As System.EventArgs) Handles MyBase.Load

    Me.Text = Application.ProductName
    Me.KeyPreview = True
    Me.MinimumSize = New Size(760, 520)
    Me.Size = My.Settings.Size
    'ShowMenu = My.Settings.ShowMenu
    ShowMenu = True
    ToolStripMenuItemSettingShowMenu.Visible = False
    AutoLoad = My.Settings.AutoLoad

    If Me.Left < GetWorkingArea(Me).Left Or
      Me.Left >= GetWorkingArea(Me).Right Then
      Me.Left = 100
    End If
    If Me.Top < GetWorkingArea(Me).Top Or
      Me.Top >= GetWorkingArea(Me).Bottom Then
      Me.Top = 100
    End If

    SaveFileDialog1.Filter =
      "タブ区切りテキスト|*.tsv|すべてのファイル|*.*"
    OpenFileDialog1.Filter = SaveFileDialog1.Filter
    With FontDialog1
      .AllowVerticalFonts = False
      .ScriptsOnly = True
      .ShowEffects = False
    End With

    Dim PhoneCategory() As String = {
      KeywordNotSet, "自宅電話", "自宅FAX", "職場電話",
      "職場FAX", "携帯電話", "電子メール"}
    ComboBoxPhone1.Items.AddRange(PhoneCategory)
    ComboBoxPhone2.Items.AddRange(PhoneCategory)
    ComboBoxPhone3.Items.AddRange(PhoneCategory)
    ComboBoxPhone4.Items.AddRange(PhoneCategory)

    SetDataGridView(DataGridViewMain)
    SetDataGridView(DataGridViewSearch)
    DataGridViewSearch.RowsDefaultCellStyle.BackColor =
      Color.Aqua
    PrintDialog1.Document = PrintDocument1
    PrintDialog1.AllowPrintToFile = False
    PrintPreviewDialog1.Document = PrintDocument1

    Initialize()
    If 1 < Environment.GetCommandLineArgs.
      Length Then
      LoadFile(System.IO.Path.GetFullPath(
        Environment.GetCommandLineArgs(1)))
    Else
      If AutoLoad And My.Settings.FileName <>
        "" Then LoadFile(My.Settings.FileName)
    End If
  End Sub

  Private Sub SetDataGridView(
    ByVal target As DataGridView)

    Dim columnName() As String = {
      "#", "姓", "名", "名前その他",
      "郵便番号", "住所1", "住所2", "住所3",
      "電話番号1", "属性", "電話番号2", "属性",
      "電話番号3", "属性", "電話番号4", "属性",
      "グループ1", "グループ2", "グループ3",
      "グループ4", "メモ"}
    ColumnCount = columnName.Count

    With target
      .Dock = DockStyle.Fill
      .ColumnCount = ColumnCount + 1
      .MultiSelect = False
      .RowHeadersVisible = False
      .ReadOnly = True
      .AllowUserToAddRows = False
      .Font = My.Settings.FontGrid
      .AutoSizeRowsMode =
        DataGridViewAutoSizeRowsMode.AllCells

      For i As Integer = 0 To ColumnCount - 1
        .Columns(i).Name = columnName(i)
      Next

      Dim s As String = My.Settings.ColumnWidth
      If s <> "" Then
        Dim columnsWidth() As String = Split(s, ",")
        For i As Integer = 0 To ColumnCount - 1
          .Columns(i).Width = columnsWidth(i)
        Next
      End If

      'For Each c In .Columns
      '  c.SortMode = DataGridViewColumnSortMode.NotSortable
      'Next
    End With
  End Sub

  Private Sub Initialize()
    EditingNewData = False
    Edited = False
    DataGridViewMain.Rows.Clear()
    ClearTabPageInput()
    FileName = ""
    TabControlMain.SelectedIndex = 0

    Dim group() As String =
      {KeywordNotSet, KeywordAddNewItem}
    ComboBoxGroup1.Items.Clear()
    ComboBoxGroup2.Items.Clear()
    ComboBoxGroup3.Items.Clear()
    ComboBoxGroup4.Items.Clear()
    ComboBoxGroup1.Items.AddRange(group)
    ComboBoxGroup2.Items.AddRange(group)
    ComboBoxGroup3.Items.AddRange(group)
    ComboBoxGroup4.Items.AddRange(group)
    ComboBoxGroup1.SelectedIndex = 0
    ComboBoxGroup2.SelectedIndex = 0
    ComboBoxGroup3.SelectedIndex = 0
    ComboBoxGroup4.SelectedIndex = 0
    ShowSearched = False
  End Sub

  Private Sub Form1_FormClosing(
    ByVal sender As System.Object,
    ByVal e As System.Windows.Forms.
    FormClosingEventArgs
    ) Handles MyBase.FormClosing

    e.Cancel = AskYouWantToCancel()
  End Sub

  Private Function AskYouWantToCancel() As Boolean
    If Edited Then
      Dim answer As DialogResult =
        MessageBox.Show("変更を保存しますか？",
          Application.ProductName,
          MessageBoxButtons.YesNoCancel,
          MessageBoxIcon.Warning)
      Select Case answer
        Case DialogResult.Yes
          If FileName = "" Then
            SaveFileDialog1.InitialDirectory =
              FilePath
            SaveFileDialog1.FileName =
              System.IO.Path.GetFileName(FileName)
            If DialogResult.OK =
              SaveFileDialog1.ShowDialog() Then
              SaveFile(SaveFileDialog1.FileName)
            Else
              Return True
            End If
          Else
            SaveFile(FileName)
          End If
          Return False
        Case DialogResult.No
          Return False
        Case DialogResult.Cancel
          Return True
      End Select
    End If
    Return False
  End Function

  Private Sub Form1_FormClosed(
    ByVal sender As System.Object,
    ByVal e As System.Windows.Forms.
    FormClosedEventArgs
    ) Handles MyBase.FormClosed

    Me.WindowState = FormWindowState.Normal
    My.Settings.Size = Me.Size
    Dim s As String = ""
    If ShowSearched Then CopyGridSetting(
      DataGridViewSearch, DataGridViewMain)
    For i As Integer = 0 To ColumnCount - 1
      s &= DataGridViewMain.Columns(i).Width
      If i <> ColumnCount - 1 Then s &= ","
    Next
    My.Settings.ColumnWidth = s
    My.Settings.FileName = FileName
    My.Settings.FontGrid = DataGridViewMain.Font
  End Sub

  Private Sub Form1_KeyDown(ByVal sender As System.Object,
    ByVal e As System.Windows.Forms.KeyEventArgs
    ) Handles MyBase.KeyDown

    If e.Alt Then MenuStrip1.Visible = True
  End Sub

  Private Sub MenuStrip1_MenuDeactivate(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles MenuStrip1.MenuDeactivate

    If Not ShowMenu Then MenuStrip1.Visible = False
  End Sub

  Private Sub ToolStripMenuItemFileExit_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFileExit.Click

    Application.Exit()
  End Sub

  Private Sub ToolStripMenuItemSettingShowMenu_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSettingShowMenu.Click

    ShowMenu = Not ShowMenu
  End Sub

  Private Sub ToolStripMenuItemHelpReadme_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemHelpReadme.Click

    Dim s As String = System.IO.Path.GetDirectoryName(
      Application.ExecutablePath)
    If Not s.EndsWith("\") Then s &= "\"
    s &= "readme.txt"
    If System.IO.File.Exists(s) Then
      System.Diagnostics.Process.Start(s)
    Else
      MessageBox.Show(s & "が見付かりません",
        Application.ProductName, MessageBoxButtons.OK,
        MessageBoxIcon.Stop)
    End If
  End Sub

  Private Sub ToolStripMenuItemHelpWeb_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemHelpWeb.Click

    System.Diagnostics.Process.Start(
      "http://software.nikkeibp.co.jp/")
  End Sub

  Private Sub ToolStripMenuItemHelpVersion_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemHelpVersion.Click

    Dim s As String
    s = Application.ProductName & " " &
      Application.ProductVersion
    s &= "（2010/11/18）" & vbCrLf
    s &= My.Application.Info.Copyright & vbCrLf & vbCrLf

    s &= "実行ファイル：" & Application.ExecutablePath & "（"
    If Environment.Is64BitProcess Then
      s &= "64"
    Else
      s &= "32"
    End If
    s &= "ビットのプロセスとして稼働）" & vbCrLf

    s &= ".NET Framework（ビルド時）：" &
      System.Reflection.Assembly.GetExecutingAssembly().
      ImageRuntimeVersion & vbCrLf
    s &= ".NET Framework（現在）：" &
      System.Runtime.InteropServices.RuntimeEnvironment.
      GetSystemVersion & vbCrLf

    s &= "オペレーティングシステム：" &
      My.Computer.Info.OSFullName &
      My.Computer.Info.OSVersion & " "
    If Environment.Is64BitOperatingSystem Then
      s &= "64"
    Else
      s &= "32"
    End If
    s &= "ビット"

    MessageBox.Show(s, "バージョン情報")
  End Sub

  Private Sub ToolStripMenuItemFileNew_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFileNew.Click

    If Not AskYouWantToCancel() Then Initialize()
  End Sub

  Private Sub ToolStripMenuItemFileOpen_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFileOpen.Click

    If AskYouWantToCancel() Then Exit Sub
    Initialize()
    OpenFileDialog1.InitialDirectory = FilePath
    OpenFileDialog1.FileName = ""
    If DialogResult.OK =
      OpenFileDialog1.ShowDialog() Then _
      LoadFile(OpenFileDialog1.FileName)
  End Sub

  Private Sub LoadFile(ByVal fileName As String)
    Dim columns() As String
    Dim s As String = System.IO.File.ReadAllText(
      fileName, System.Text.Encoding.
      GetEncoding("Shift_JIS"))
    Dim lines() As String = Split(s, vbCrLf)

    DataGridViewMain.Rows.Clear()
    For Each line As String In lines
      If line <> "" Then
        columns = Split(line, vbTab)
        s = ""
        For i As Integer =
          ColumnCount - 1 To columns.
          GetUpperBound(0)
          s &= columns(i) & vbCrLf
        Next
        ReDim Preserve columns(ColumnCount - 1)
        columns(ColumnCount - 1) = s
        DataGridViewMain.Rows.Add(columns)
      End If
    Next
    Me.FileName = fileName
    InitializeComboBoxGroup()
  End Sub

  Private Sub InitializeComboBoxGroup()
    Dim group As ArrayList = New ArrayList()
    group.Add(KeywordAddNewItem)
    Dim s As String

    For i As Integer = 0 To DataGridViewMain.
      RowCount - 1
      For j As Integer = 16 To 19
        s = DataGridViewMain(j, i).Value
        If (s <> "") And (-1 = group.IndexOf(s)
          ) Then group.Add(s)
      Next
    Next
    group.Sort()

    ComboBoxGroup1.Items.Clear()
    ComboBoxGroup2.Items.Clear()
    ComboBoxGroup3.Items.Clear()
    ComboBoxGroup4.Items.Clear()
    Dim groupString(group.Count - 1) As String
    group.CopyTo(groupString)
    ComboBoxGroup1.Items.AddRange(groupString)
    ComboBoxGroup2.Items.AddRange(groupString)
    ComboBoxGroup3.Items.AddRange(groupString)
    ComboBoxGroup4.Items.AddRange(groupString)
  End Sub

  Private Sub ToolStripMenuItemFileSave_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFileSave.Click

    SaveFile(FileName)
  End Sub

  Private Sub ToolStripMenuItemFileSaveAs_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFileSaveAs.Click

    SaveFileDialog1.InitialDirectory = FilePath
    SaveFileDialog1.FileName =
      System.IO.Path.GetFileName(FileName)
    If DialogResult.OK =
      SaveFileDialog1.ShowDialog() Then _
      SaveFile(SaveFileDialog1.FileName)
  End Sub

  Private Sub SaveFile(ByVal fileName As String)
    Dim s As String
    Dim obj As Object
    Dim sb As New System.Text.StringBuilder()
    For j As Integer = 0 To DataGridViewMain.
      RowCount - 1
      s = ""
      For i As Integer = 0 To ColumnCount - 1
        obj = DataGridViewMain(i, j).Value
        If Not obj Is Nothing Then
          If i <> ColumnCount - 1 Then
            s &= obj.ToString() & vbTab
          Else
            s &= obj.ToString().Replace(vbCrLf,
              vbTab)
            If s.EndsWith(vbTab) Then _
              s = s.Remove(s.Length - 1)
          End If
        End If
      Next
      sb.AppendLine(s)
    Next
    System.IO.File.WriteAllText(fileName,
      sb.ToString(), System.Text.Encoding.
      GetEncoding("Shift_JIS"))
    Me.FileName = fileName
  End Sub

  Private Sub ToolStripMenuItemEdit_DropDownOpening(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemEdit.DropDownOpening

    If DataGridViewMain.RowCount > 0 Then
      ToolStripMenuItemEditDelete.Enabled = True
    Else
      ToolStripMenuItemEditDelete.Enabled = False
    End If
  End Sub

  Private Sub ToolStripMenuItemEditDelete_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemEditDelete.Click

    If DataGridViewMain.RowCount = 0 Then Exit Sub
    If ShowSearched Then CopyGridSetting(
      DataGridViewSearch, DataGridViewMain)
    Dim i As Integer =
      DataGridViewMain.CurrentRow.Index
    Dim s As String = "「#" &
      DataGridViewMain(0, i).Value & " " &
      DataGridViewMain(1, i).Value & " " &
      DataGridViewMain(2, i).Value &
      "」を削除しますか？"
    If DialogResult.Yes = MessageBox.Show(
      s, "データ削除の確認",
      MessageBoxButtons.YesNo,
      MessageBoxIcon.Question) Then

      DataGridViewMain.Rows.RemoveAt(i)
      Edited = True
      If TabControlMain.SelectedIndex = 1 Then
        If DataGridViewMain.RowCount = 0 Then
          ClearTabPageInput()
          ShowOneRow(-1)
        Else
          If i = DataGridViewMain.RowCount Then _
            i -= 1
          ShowOneRow(i)
        End If
      End If

      If ShowSearched Then
        DataGridViewSearch.Rows.RemoveAt(
          DataGridViewSearch.CurrentRow.Index)
        If DataGridViewSearch.RowCount = 0 Then _
          ReturnToDisplayAll()
      End If
    End If
  End Sub

  Sub ReturnToDisplayAll()
    MessageBox.Show(
      "検索結果がなくなったので、全データ表示に戻ります",
      Application.ProductName, MessageBoxButtons.OK,
      MessageBoxIcon.Asterisk)
    ShowSearched = False
    TabControlMain.SelectedIndex = 0
  End Sub

  Private Sub ToolStripMenuItemSettingAutoLoad_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSettingAutoLoad.Click

    AutoLoad = Not AutoLoad
  End Sub

  Private Sub ToolStripMenuItemSettingFont_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSettingFont.Click

    FontDialog1.Font = DataGridViewMain.Font
    If FontDialog1.ShowDialog() Then _
      DataGridViewMain.Font = FontDialog1.Font
  End Sub

  Private Sub TabPageInput_Enter(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles TabPageInput.Enter

    If ShowSearched Then CopyGridSetting(
      DataGridViewSearch, DataGridViewMain)
    ShowOneRow(GetRowIndex)
  End Sub

  Private Function GetRowIndex()
    If DataGridViewMain.RowCount = 0 Then
      Return -1
    Else
      Return DataGridViewMain.CurrentRow.Index
    End If
  End Function

  Private Sub ShowOneRow(ByVal index As Integer)
    If index = -1 Then
      LabelId.Text = "#0"
      EnableTextBoxAndComboBox(False)
      SetMoveButtons(False)
      ButtonOverwrite.Enabled = False
      ButtonSaveAsNew.Enabled = False
    Else
      With DataGridViewMain.Rows(index)
        LabelId.Text = "#" & .Cells(0).Value
        TextBoxNameLast.Text = .Cells(1).Value
        TextBoxNameFirst.Text = .Cells(2).Value
        TextBoxNameOthers.Text = .Cells(3).Value
        TextBoxPostalNumber.Text = .Cells(4).Value
        TextBoxAddress1.Text = .Cells(5).Value
        TextBoxAddress2.Text = .Cells(6).Value
        TextBoxAddress3.Text = .Cells(7).Value
        TextBoxPhone1.Text = .Cells(8).Value
        ComboBoxPhone1.SelectedItem = .Cells(9).Value
        TextBoxPhone2.Text = .Cells(10).Value
        ComboBoxPhone2.SelectedItem = .Cells(11).Value
        TextBoxPhone3.Text = .Cells(12).Value
        ComboBoxPhone3.SelectedItem = .Cells(13).Value
        TextBoxPhone4.Text = .Cells(14).Value
        ComboBoxPhone4.SelectedItem = .Cells(15).Value
        ComboBoxGroup1.SelectedItem = .Cells(16).Value
        ComboBoxGroup2.SelectedItem = .Cells(17).Value
        ComboBoxGroup3.SelectedItem = .Cells(18).Value
        ComboBoxGroup4.SelectedItem = .Cells(19).Value
        TextBoxMemo.Text = .Cells(20).Value
      End With
      SetMoveButtons(True)
      TextBoxNameLast.Focus()
    End If
  End Sub

  Private Sub ButtonFirst_Click(ByVal sender As System.Object,
    ByVal e As System.EventArgs) Handles ButtonFirst.Click

    DataGridViewMain(0, 0).Selected = True
    ShowOneRow(0)
  End Sub

  Private Sub ButtonPrevious_Click(ByVal sender As System.Object,
    ByVal e As System.EventArgs) Handles ButtonPrevious.Click

    Dim i As Integer = DataGridViewMain.CurrentRow.Index
    If i <> 0 Then i -= 1
    DataGridViewMain(0, i).Selected = True
    ShowOneRow(i)
  End Sub

  Private Sub ButtonNext_Click(ByVal sender As System.Object,
    ByVal e As System.EventArgs) Handles ButtonNext.Click

    Dim i As Integer = DataGridViewMain.CurrentRow.Index
    If i < DataGridViewMain.RowCount - 1 Then i += 1
    DataGridViewMain(0, i).Selected = True
    ShowOneRow(i)
  End Sub

  Private Sub ButtonLast_Click(ByVal sender As System.Object,
  ByVal e As System.EventArgs) Handles ButtonLast.Click

    Dim i As Integer = DataGridViewMain.RowCount - 1
    DataGridViewMain(0, i).Selected = True
    ShowOneRow(i)
  End Sub

  Private Sub DataGridViewMain_DoubleClick(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles DataGridViewMain.DoubleClick, DataGridViewSearch.DoubleClick

    TabControlMain.SelectedIndex = 1
  End Sub

  Private Sub ComboBoxGroup_SelectedIndexChanged(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ComboBoxGroup1.SelectedIndexChanged,
    ComboBoxGroup4.SelectedIndexChanged,
    ComboBoxGroup3.SelectedIndexChanged,
    ComboBoxGroup2.SelectedIndexChanged

    Dim s As String
    Dim comboBox As ComboBox = CType(sender, ComboBox)
    If comboBox.SelectedItem = KeywordAddNewItem Then
      s = InputBox("新しいグループ名を入力してください",
        "新規グループ名入力")
      If s = "" Or s = KeywordAddNewItem Then
        comboBox.SelectedItem = KeywordNotSet
      Else
        If -1 = comboBox.Items.IndexOf(s) Then
          ComboBoxGroup1.Items.Add(s)
          ComboBoxGroup2.Items.Add(s)
          ComboBoxGroup3.Items.Add(s)
          ComboBoxGroup4.Items.Add(s)
        End If
        comboBox.SelectedItem = s
      End If
    End If
  End Sub

  Private Sub ButtonNew_Click(ByVal sender As System.Object,
    ByVal e As System.EventArgs) Handles ButtonNew.Click

    EditingNewData = Not EditingNewData
  End Sub

  Private Sub TabControlMain_Deselecting(
    ByVal sender As System.Object,
    ByVal e As System.Windows.Forms.TabControlCancelEventArgs
    ) Handles TabControlMain.Deselecting

    If EditingNewData Then e.Cancel = True
  End Sub

  Private Sub ButtonSaveAsNew_Click(ByVal sender As System.Object,
    ByVal e As System.EventArgs) Handles ButtonSaveAsNew.Click

    If Not EditingNewData Then _
      LabelId.Text = "#" & GetNewDataIndex()
    WriteDataToGrid(DataGridViewMain.RowCount)
  End Sub

  Private Sub WriteDataToGrid(ByVal index As Integer)
    If DataGridViewMain.RowCount = index Then _
      DataGridViewMain.Rows.Add()
    With DataGridViewMain.Rows(index)
      .Cells(0).Value = LabelId.Text.Substring(1)
      .Cells(1).Value = TextBoxNameLast.Text
      .Cells(2).Value = TextBoxNameFirst.Text
      .Cells(3).Value = TextBoxNameOthers.Text
      .Cells(4).Value = TextBoxPostalNumber.Text
      .Cells(5).Value = TextBoxAddress1.Text
      .Cells(6).Value = TextBoxAddress2.Text
      .Cells(7).Value = TextBoxAddress3.Text
      .Cells(8).Value = TextBoxPhone1.Text
      .Cells(9).Value = ComboBoxPhone1.SelectedItem
      .Cells(10).Value = TextBoxPhone2.Text
      .Cells(11).Value = ComboBoxPhone2.SelectedItem
      .Cells(12).Value = TextBoxPhone3.Text
      .Cells(13).Value = ComboBoxPhone3.SelectedItem
      .Cells(14).Value = TextBoxPhone4.Text
      .Cells(15).Value = ComboBoxPhone4.SelectedItem
      .Cells(16).Value = ComboBoxGroup1.SelectedItem
      .Cells(17).Value = ComboBoxGroup2.SelectedItem
      .Cells(18).Value = ComboBoxGroup3.SelectedItem
      .Cells(19).Value = ComboBoxGroup4.SelectedItem
      .Cells(20).Value = TextBoxMemo.Text
    End With
    DataGridViewMain(0, index).Selected = True
    EditingNewData = False
    Edited = True

    If ShowSearched Then _
      If Not Search(LastSearch.Mode, LastSearch.Keyword) Then _
        ReturnToDisplayAll()
  End Sub

  Private Sub ButtonOverwrite_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ButtonOverwrite.Click

    WriteDataToGrid(DataGridViewMain.CurrentRow.Index)
  End Sub

  Private Sub ToolStripMenuItemSearch_DropDownOpening(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSearch.DropDownOpening

    ToolStripMenuItemSearchDisplayAll.Enabled =
      ShowSearched
    Dim b As Boolean =
      DataGridViewMain.RowCount > 0
    ToolStripMenuItemSearchId.Enabled = b
    ToolStripMenuItemSearchGroup.Enabled = b
    ToolStripMenuItemSearchFreeWord.Enabled = b
  End Sub

  Private Sub ToolStripMenuItemSearchId_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSearchId.Click

    Dim found As Boolean = False
    Dim id As Integer
    Dim s As String = InputBox(
      "検索したいID番号を入力してください",
      "ID番号入力")

    If s <> "" Then
      Try
        id = Int32.Parse(s)
      Catch ex As Exception
        ShowError("「" & s &
          "」は数値ではありません")
        Exit Sub
      End Try
      If id < 1 Then
        ShowError("ID番号は1以上です")
        Exit Sub
      End If

      If Search(SearchMode.Id, id) Then
        ShowSearched = True
      Else
        ShowError(
          "そのID番号のデータはありません")
      End If
    End If
  End Sub

  Sub ShowError(ByVal s As String)
    MessageBox.Show(s, "エラー",
      MessageBoxButtons.OK,
      MessageBoxIcon.Error)
  End Sub

  Function Search(ByVal mode As SearchMode,
    ByVal keyword As String) As Boolean

    Dim result As Boolean = False
    Select Case mode
      Case SearchMode.Id
        For i As Integer = 0 To _
          DataGridViewMain.RowCount - 1
          If keyword =
            DataGridViewMain(0, i).Value Then
            DataGridViewSearch.Rows.Clear()
            result = CopyOneRow(i)
            Exit For
          End If
        Next

      Case SearchMode.Group
        DataGridViewSearch.Rows.Clear()
        For i As Integer = 0 To _
          DataGridViewMain.RowCount - 1
          For j As Integer = 16 To 19
            If keyword =
              DataGridViewMain(j, i).Value Then
              result = CopyOneRow(i)
              Exit For
            End If
          Next
        Next

      Case SearchMode.FreeWord
        Dim id As Integer = 0
        DataGridViewSearch.Rows.Clear()
        For i As Integer = 0 To _
          DataGridViewMain.RowCount - 1
          For j As Integer = 0 To _
            ColumnCount - 1
            If -1 <> DataGridViewMain(j, i).Value.
              ToString().IndexOf(keyword) Then
              result = CopyOneRow(i)
              Exit For
            End If
          Next
        Next
    End Select

    If result Then
      LastSearch.Mode = mode
      LastSearch.Keyword = keyword
    End If
    Return result
  End Function

  Function CopyOneRow(
    ByVal index As Integer) As Boolean

    DataGridViewSearch.Rows.Add()
    For i As Integer = 0 To ColumnCount - 1
      DataGridViewSearch(i,
        DataGridViewSearch.RowCount - 1).Value =
        DataGridViewMain(i, index).Value
    Next
    Return True
  End Function

  Private Sub ToolStripMenuItemSearchDisplayAll_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSearchDisplayAll.Click

    ShowSearched = False
  End Sub

  Private Sub ToolStripMenuItemSearchGroup_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSearchGroup.Click

    If DialogResult.OK =
      FormSelectGroup.ShowDialog() Then
      If Search(SearchMode.Group,
        FormSelectGroup.Group) Then
        ShowSearched = True
      Else
        ShowError("「" &
          FormSelectGroup.Group &
          "」のデータはありません")
        ShowSearched = False
      End If
    End If
  End Sub

  Private Sub ToolStripMenuItemSearchFreeWord_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSearchFreeWord.Click

    Dim s As String =
      InputBox("検索したい単語を入力してください",
      "自由語検索")
    If s <> "" Then
      If Search(SearchMode.FreeWord, s) Then
        ShowSearched = True
      Else
        ShowError("「" & s &
          "」を含むデータはありません")
        ShowSearched = False
      End If
    End If
  End Sub

  Private PrintString As ArrayList = New ArrayList
  Private PrintStringIndex As Integer

  Private Sub ToolStripMenuItemFilePrint_Click(
    ByVal sender As System.Object, ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFilePrint.Click

    If DialogResult.OK = PrintDialog1.ShowDialog Then
      SetPrintDocument()
      PrintDocument1.Print()
    End If
  End Sub

  Private Sub SetPrintDocument()
    Dim sb As New System.Text.StringBuilder()
    Dim s As String
    Dim targetGrid As DataGridView = GetTargetGrid()

    PrintString.Clear()
    For i As Integer = 0 To targetGrid.RowCount - 1
      s = targetGrid(4, i).Value.ToString()
      If s <> "" Then _
        sb.AppendLine(Mid(s, 1, 3) & "-" & Mid(s, 4))

      For j As Integer = 5 To 7
        s = targetGrid(j, i).Value.ToString()
        If s <> "" Then sb.AppendLine(s)
      Next

      s = targetGrid(1, i).Value.ToString() & " " &
        targetGrid(2, i).Value.ToString() & " 様"
      sb.AppendLine(s)
      PrintString.Add(sb.ToString())
      sb.Clear()
    Next

    PrintStringIndex = 0
    PrintDocument1.DocumentName = FileName
  End Sub

  Function GetTargetGrid() As DataGridView
    If ShowSearched Then
      Return DataGridViewSearch
    Else
      Return DataGridViewMain
    End If
  End Function

  Private Sub PrintDocument1_PrintPage(
    ByVal sender As System.Object,
    ByVal e As System.Drawing.Printing.PrintPageEventArgs
    ) Handles PrintDocument1.PrintPage

    Const unit As Double = 0.254
    Const sheetWidth As Integer = 210 / unit
    Const sheetHeight As Integer = 297 / unit
    Const divideHorizontal As Integer = 3
    Const divideVertical As Integer = 7
    Const offsetX As Integer = -3 / unit
    Const offsetY As Integer = -4 / unit
    Const margin As Integer = 7 / unit

    Dim rect As Rectangle
    Dim areaHeight As Integer =
      sheetHeight \ divideVertical - margin * 2
    Dim areaWidth As Integer =
      sheetWidth \ divideHorizontal - margin * 2

    For i As Integer = 0 To divideVertical - 1
      For j As Integer = 0 To divideHorizontal - 1
        rect = New Rectangle(offsetX +
          margin + (j * (areaWidth + margin * 2)),
          offsetY + margin + (i *
          (areaHeight + margin * 2)),
          areaWidth, areaHeight)
        'e.Graphics.DrawRectangle(Pens.Black, rect)
        e.Graphics.DrawString(
          PrintString(PrintStringIndex),
          My.Settings.FontPrint,
          Brushes.Black, rect,
          StringFormat.GenericTypographic)

        PrintStringIndex += 1
        If PrintStringIndex >= PrintString.Count Then
          e.HasMorePages = False
          PrintStringIndex = 0
          Exit Sub
        End If
      Next
    Next
    e.HasMorePages = True
  End Sub

  Private Sub ToolStripMenuItemFilePrintPreview_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFilePrintPreview.Click

    PrintPreviewDialog1.Width = Me.Width - 200
    PrintPreviewDialog1.Height = Me.Height
    SetPrintDocument()
    PrintPreviewDialog1.ShowDialog()
  End Sub

  Private Sub ToolStripMenuItemSettingPrintFont_Click(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemSettingPrintFont.Click

    FontDialog1.Font = My.Settings.FontPrint
    If FontDialog1.ShowDialog() Then _
      My.Settings.FontPrint = FontDialog1.Font
  End Sub

  Private Sub ToolStripMenuItemFile_DropDownOpening(
    ByVal sender As System.Object,
    ByVal e As System.EventArgs
    ) Handles ToolStripMenuItemFile.DropDownOpening

    If GetTargetGrid().RowCount = 0 Then
      ToolStripMenuItemFilePrintPreview.Enabled = False
      ToolStripMenuItemFilePrint.Enabled = False
    Else
      ToolStripMenuItemFilePrintPreview.Enabled = True
      ToolStripMenuItemFilePrint.Enabled = True
    End If
  End Sub
End Class
